@@ -89,6 +89,9 @@ private:
 
 void tst_qtiff::initTestCase()
 {
+    if (!QImageReader::supportedImageFormats().contains("tiff"))
+        QSKIP("The image format handler is not installed.");
+
     prefix = ":/tiff/";
 }
 
@@ -343,6 +346,7 @@ void tst_qtiff::writeImage()
         QImageWriter writer(&buf, format);
         QVERIFY(writer.write(image));
     }
+    image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
 
     QImage image2;
     {
